/*
 * Motf.java
 * Copyright 2005, Hiroki Arimura, Takeaki Uno, all rights reserved.
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Efficient enumeration of maximal motifs in a sequence,
 * Hiroki Arimura and Takeaki Uno, Manuscript, Division of
 * Computer Science, * IST, Hokkaido University, 9 April 2005;
 * Modified 18 April 2005.  
 * <http:>
 *
 * This software may be used freely for any purpose. However, when
 * distributed, the original source must be clearly stated, and, 
 * when the source code is distributed, the copiright notice must be
 * retained and any alterations in the code must be clearly marked. 
 * No warranty is given regarding the quality of this software.
 */

package motf; 
import motf.*;
import java.io.*;
import java.util.*;

/**
 * DocSet.java
 * Created: 08 June 2005
 *
 * @author <a href="mailto:arim@ist.hokudai.ac.jp">Hiroki Arimura</a>
 * @version $Revision: 1.7 $
 */
public abstract class DocSet {
	/*
	 * 
	 */

    /** Java  DEBUG ͡ */
	static boolean useDebug  	= false;

	/** ɸϤɽ魯ޥɰʸ */
	public static final String STDIN_STRING = "-"; 

    /** EOFʸ͡ */
	static int EOF 	= -1;
	static int NL  	= '\n';

	/*
	 * Сե
	 */

	ArrayList<Sequence> docList = new ArrayList<Sequence>();

	DocStat docStat = new DocStat(this);

	/*
	 * Сեɡץʥ
	 */

    /** å٤Ȥʸ٤Ѥ뤫ɤ򼨤 */
	boolean useDocCount = false;

	public void setUseDocCount(boolean flag) { this.useDocCount = flag; }
	/*
	 * ³
	 */

	public int size() {
		return docList.size();
	}

	public Sequence get(int sid) {
		return docList.get(sid);
	}

	/*
	 * ե
	 */

	public void readFiles(List files)
	{
		for (int i = 0; i < files.size(); i++) {
			String filename = (String) files.get(i);
			if (filename.equals(DocSet.STDIN_STRING)) {
				readFile(System.in);
				return; //ɤ߹ĤΰϼΤƤ롥
			} else {
				readFile(filename);
			}
		}
	}

	/** åѡե뤫ɤ߹ࡥ*/
	public void readFile(String filename)
	{
		Log.log("@log Read_File: " + filename + "\n");
		try {
			FileReader fr = new FileReader(filename); //for debug
			Log.println("@log Encoding: " + fr.getEncoding());
			readFile(fr);
			this.docStat.isUptoDate = false;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/** åѡե륹ȥ꡼फɤ߹ࡥ*/
	public void readFile(InputStream is)
	{
		Log.log("@log Read_File: InputStream("+is+")\n");
		try {
			readFile(new BufferedReader(new InputStreamReader(is)));
			this.docStat.isUptoDate = false;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/** ꤵ줿ȥ꡼फɤ߹ */
	abstract void readFile(Reader in); 


	int getTotalNumOfLetters() {
		return docStat.getTotalNumOfLetters(); 
	}

    /**
     * ǾݡȤѡɽниѴ롥
     */
    public int convertSupRatioToAbsolute(double ratio)
    {
		int totalNumber;

		docStat.ensureUptoDate();
		if (this.useDocCount)
			totalNumber = this.docList.size();
		else
			totalNumber = this.docStat.getTotalNumOfLetters(); 

        int minsup = (int) Math.ceil(totalNumber * ratio);
		if (minsup <= 1) {
			minsup = Math.max(minsup, 2); 
			Log.print("@log Warning: MinSupCount must be >= 2. "+
					  "Automatically changed to: " + minsup + "\n");
		}
        return minsup; 
    }

	public void printStat(PrintStream out) {
		this.docStat.print(out);
	}

	/** ꤵ줿ȥ꡼˽Ϥ */
	public void writeFile(PrintStream out) {
		this.docStat.ensureUptoDate(); 
		out.println("@docset"+
					" "+docList.size()+
					" "+this.docStat.getTotalNumOfLetters());
		for (int i = 0; i < docList.size(); i++) {
			Sequence doc = docList.get(i);
			out.println(i+": "+doc.toString());
		}
	}

	/***************************************************
	 * 
	 ***************************************************/
}


/**ʸ񽸹פ */
class DocStat {

	DocSet docSet = null;
	public boolean isUptoDate = false; //the status of update 

	/* ץѥ᡼ */
	int num = 0;
	int lenAve = 0;	//ʸĹʿ
	int lenMax = 0;	//ʸĹκ
	int lenMin = -1;//ʸĹκǾ
	int lenVar = 0;	//ʸĹʬ
	int lenSum = 0; //ɬܡʸĹ¡ᡡʸ
	int lenSumSqr = 0;

	DocStat(DocSet docSet) {
		this.docSet = docSet; 
	}

	public int getTotalNumOfLetters() {
		this.ensureUptoDate();
		return lenSum;
	} 

	public void setTouched() { isUptoDate = false; } 

	public void ensureUptoDate() {
		if (!this.isUptoDate)
			update();
	}

	/** ʸ񽸹פ */
	public void update() {

		ArrayList<Sequence> docList = docSet.docList; 

		if (docList.size() == 0) {
			return;
		}

		//scan lines
		for (Sequence doc : docList) {
			int len = doc.size();
			lenMax = Math.max(lenMax, len);
			if (lenMin < 0) lenMin = len;
			else    		lenMin = Math.min(lenMin, len); 
			lenSum = lenSum + len; 
			lenSumSqr = lenSumSqr + len*len; 
			num++; 
		}
		//post
		lenAve = lenSum / num;

		/* Variance of random variable Xs = (X_1,...,X_n): 
		 * VAR[X] = (1/n)* SUM[(X - (1/n)*SUM[X])^2] 
		 * = (1/n)*{ SUM[X*X] - (1/n)*SUM[X]*SUM[X]} */

		lenVar = ((lenSumSqr * num) - (lenSum * lenSum)) / (num * num);

		this.isUptoDate = true;
	}

	/** ʸ񽸹פ */
	public void print(PrintStream out) {		

		if (this.docSet.docList.size() == 0) {
			out.println("@stat numOfDocs: "+0);
			return;
		}

		ensureUptoDate(); 

		//print
		out.println("@stat numberOfDocumns: "+num);
		out.println("@stat numberOfletters: "+lenSum+" (chars)"+"\t"+
					String.format("%.3f", ((double)lenSum/1000000.0)) +" (MB)");
		out.println("@stat on sequences...");
		out.println("@stat minLength: "+lenMin);
		out.println("@stat maxLength: "+lenMax);
		out.println("@stat aveLength: "+lenAve);
		out.println("@stat varLength: "+
					String.format("%g", Math.sqrt(lenVar)));
	}
}


/* EOF */
